//=============================================================================
//
//  CLASS Ray - IMPLEMENTATION
//
//=============================================================================
#define RAYTRACER_RAY_CPP

//== INCLUDES =================================================================
#include "Ray.h"

//== IMPLEMENTATION ==========================================================
Ray Ray::reflectedRay(
    const VectorType3   &v3Point,   
    const VectorType3   &v3Normal )  const
{
    /*
		Berechnung des reflektierten Strahls am Punkt v3Point um die
		Normale v3Normal.		
	*/
    Ray result;
    /*~~~~~~~*/

	//We don't assume normalized direction
	VectorType3 dirNormalized = m_v3Dir;
	dirNormalized.normalize();
	//...but normalized normal
	result.setDir( dirNormalized- (v3Normal*(2*(dirNormalized|v3Normal))) );
	result.setOrigin( v3Point);	  
	return result;
};

//-----------------------------------------------------------------------------
bool Ray::refractedRay(
    const VectorType3   &v3Point,           
    const VectorType3   &v3Normal,          
    double            rRefractiveIndex,   
    Ray                 &clRefractedRay )  const
{

    /*
		Berechnung des gebrochenen Strahls am Punkt v3Point mit Normale v3Normal 
		und Brechungsindex rRefractiveIndex. Geben sie false zurck,
		falls Totalreflektion eintritt.

		Hinweis: Es wird davon ausgegangen, dass der bergang stets zwischen
		dem Material mit dem bergebenen Brechungsindex rRefractiveIndex und
		Luft (Brechungsindex ~1) stattfindet.
	*/
	
	//We don't assume normalized direction
	VectorType3 dirNormalized = m_v3Dir;
	dirNormalized.normalize();
	//...but normalized normal	
	double cosTheta = -dirNormalized | v3Normal;
	rRefractiveIndex = 1/rRefractiveIndex;
    double rSqrt = 1-rRefractiveIndex*rRefractiveIndex*(1-cosTheta*cosTheta);
	if (rSqrt<0) 
	{
		return false;
	}
	else
	{
		rSqrt = sqrt(rSqrt);
		clRefractedRay.setDir(rRefractiveIndex*dirNormalized +
								(rRefractiveIndex*cosTheta-rSqrt)*v3Normal); 

		clRefractedRay.setOrigin( v3Point );	
		return true;
	}

    return true;
};